-- Macroception, by pankker / Maunotavast Zenedar-EU
-- Macroception is hereby placed in the Public Domain.

-- Some CFG variables here for you guys to configure to your liking!

local BUTTON_NAME = "m"       -- Base name of our button!

-- Do not use square brackets [] or semicolon ; for the below options.
-- They will interfere with SecureCmdOptionParse and stuff won't work :P

local MOUSE_DELIM = ":"       -- Separator between macro name and mouse button
local MACRO_DELIM = ","       -- Separator for listing multiple macros
local SPACE_CHAR = ""         -- Substitute character for space (so you'll be able to specify macros with spaces in their name)

local NUM_EXTRA_BUTTONS = 9   -- Number of additional macro buttons to create.

--[=[
   MACROCEPTION!!!

   WHAT DOES IT DO?
      Macroception allows you to call a macro from within another macro.
      
      Currently only default UI macros can be called.
      Support for different custom macro AddOns is planned, though! Well, maybe.
      
   HOW DO I USE IT?
      /click m macroName:mouseButton
      
      (That is a lowercase m, mind you! It's case-sensitive.)
      
      macroName: 
         The name of the macro to call. If this is a number, it will call the macro by its ID instead
         (which is not recommended as these tend to change, I think?).
                
      mouseButton: 
         Simulates this button in the macro (for [btn] conditional).
      
         The [btn:X] conditional works with numbers up to 31,
         The button names for these numbers are (in order):
         LeftButton, RightButton, MiddleButton, Button4, Button5, ..., Button31
         You can also just give a number between 1-31 and it'll be converted to
         the correct button name.
         
         If a non-mousebutton string is supplied, [btn:string] will evaluate true,
         e.g.: "/click m myMacro:Heyooo" would call myMacro where
         any [btn:heyooo] (case-insensitive) conditionals would be true
                    
      --------------------------------------
           
      Important: 
         There may not be any space characters in the arguments. 
         If there are spaces in your macro names, remove them.
                 
         (Or, alternatively, you can specify a substitute space character by editing... THE SOURCE.)
      
      Examples:
         # Call macroA with left click:
         /click m macroA
         /click m macroA:LeftButton
         /click m macroA:1
         
         # Call macroB with middle click:
         /click m macroB:MiddleButton
         /click m macroB:3      
      
         # You can also run multiple macros at once by using a comma separator.
         /click m macroA,macroB,macroC,macroD
      
      Chaining?:
      
      It is not possible for a button to click itself, or any other button previously found in the "click stack".
      This is a restriction (or safety feature :P) by Blizzard to prevent infinite loops from happening.
      For this reason, the following won't work when using MacroA:
      
         MacroA:
         /click m MacroB
         
         MacroB:
         /click m MacroC
         
         MacroC:
         /click m MacroD
         
         MacroD:
         /say Yay!
      
      To alleviate this a bit, Macroception will provide a number of additional macro buttons (9 by default!)
      that can be used when you need to do chaining. The buttons are named m1, m2, m3, m4... m9.
      
      To make the above example work:
      
         MacroA:
         /click m MacroB
         
         MacroB:
         /click m1 MacroC
         
         MacroC:
         /click m2 MacroD
         
         MacroD:
         /say Yay!
      
      
   WHY DOES THIS WORK? (technical)
      Well, the /click command allows us to specify what mousebutton is used when clicking a button.
      However it doesn't HAVE to be sensible mousebutton name. It can be ANYTHING. Well, excluding space characters, of course :P
      The button "name" can be grabbed in RestrictedEnvironment, where we can then parse it and do all sorts of things.  
      Like, choose a macro to run!
--]=] 

local addonName, t = ...

t.env = t.env or CreateFrame("frame", nil, nil, "SecureHandlerBaseTemplate")
t.btns = t.btns or {}

local function EscapePattern(str)
   return str:gsub(('(['..("%^$().[]*+-?"):gsub("(.)", "%%%1")..'])'), "%%%1")
end

t.env:Execute(([[
   SPACE_CHAR = "%s"
   MOUSE_DELIM = "%s"
   MACRO_PATTERN = "[^%s]+"
   BUTTONS = newtable("LeftButton", "RightButton", "MiddleButton")
   macroList = newtable()
]]):format(EscapePattern(SPACE_CHAR or ""), MOUSE_DELIM or ":", EscapePattern(MACRO_DELIM or ",")))

local function CreateButton(name)
   assert(type(name)=="string" and name ~= "", "Illegal button name")
  
   -- The master button! We'll parse the whole command here to get separate macros, 
   -- and we'll then /click the executor for each macro given.
   local b = t.btns[name] or CreateFrame("Button", name, nil, "SecureActionButtonTemplate")
   b:RegisterForClicks("AnyDown")
   b:SetAttribute("type", "macro")
   t.btns[name] = b

   SecureHandlerUnwrapScript(b, "OnClick")
   SecureHandlerWrapScript(b, "OnClick", t.env, [[
   local numMacros = 0
   for macro in button:gmatch(MACRO_PATTERN) do
      numMacros = numMacros + 1
      macroList[numMacros] = macro
   end

   local macroText = ""
   local executor = self:GetName() .. "x"
   for i = 1, numMacros do
      macroText = macroText .. "/click " .. executor .. " " .. macroList[i] .. "\10"
   end

   self:SetAttribute("macrotext", macroText)
   ]])


   -- Executor for each button that will run the actual macros.
   local x = b.executor or CreateFrame("Button", name.."x", nil, "SecureActionButtonTemplate")
   x:RegisterForClicks("AnyDown")
   x:SetAttribute("type", "macro")

   SecureHandlerUnwrapScript(x, "OnClick")
   SecureHandlerWrapScript(x, "OnClick", t.env, [[
   local macroName, newButton = strsplit(MOUSE_DELIM, button)
   if SPACE_CHAR ~= "" then
      macroName = macroName:gsub(SPACE_CHAR, " ")
   end
   self:SetAttribute("macro", macroName)

   local buttonNum = tonumber(newButton)
   if (buttonNum or 0) > 0 then
      newButton = BUTTONS[buttonNum] or (buttonNum<32 and "Button" or "b")..buttonNum
   end

   return newButton or "LeftButton"
   ]])

   b.executor = x
end

CreateButton(BUTTON_NAME)

for i = 1, NUM_EXTRA_BUTTONS do
  CreateButton(BUTTON_NAME .. i)
end
